package chargePoint;

import java.util.Scanner;

public class GameOfLife {

	public static void main(String[] args) {
		int[][] universe = {{0,1,0},{0,0,1},{1,1,1},{0,0,0}}; // Stabilizes after Generation 6
//				int[][] array = {{0,1,0},{1,0,1},{1,0,1},{0,1,0}}; //BeeHive - Stable
//				int[][] array = {{0,1,0},{0,1,0},{0,1,0},{0,1,0}}; //Blinker - Stabilizes after Generation 3
		int rows = universe.length;
		int cols = universe[0].length;
		int generation = 1;

		System.out.printf("Generation %s \n", generation);
		for(int i = 0; i < rows; i++) {
			for(int j = 0; j < cols; j++) {
				System.out.print(universe[i][j]+"  ");
			}
			System.out.println();
		}
		boolean moveToNextGen = false;
		Scanner in = new Scanner(System.in);
		int changes = 0;
		int prevChange = 0;
		do {
			System.out.println();
			for(int i = 0; i < rows; i++) {
				for(int j = 0; j < cols; j++) {
					int count = countActiveNeighbours(i, j, universe);
					changes += changeState(count, i, j, universe);
				}
			}

			if(changes == prevChange) {
				System.out.println("Universe is stable now, Live happily...");
				moveToNextGen = false;
			} else {
				System.out.printf("Generation %s \n", ++generation);
				for(int i = 0; i < rows; i++) {
					for(int j = 0; j < cols; j++) {
						System.out.print((universe[i][j]%=2)+"  ");
					}
					System.out.println();
				}
				System.out.print("Want to move ahead? (Y/N) : ");
				String ans = in.nextLine();
				moveToNextGen = "Y".equalsIgnoreCase(ans);
			}
			prevChange = changes;
		} while(moveToNextGen);
		in.close();
	}

	/**
	 * Changes the state of given cell in the given matrix, depending on the rules.
	 * @param count
	 * @param rowIndex
	 * @param columnIndex
	 * @param matrix
	 * @return Returns 1 if state is changed else returns 0
	 */
	private static int changeState(int count, int rowIndex, int columnIndex, int[][] matrix) {
		int curStatus = matrix[rowIndex][columnIndex];
		int changed = 0;
		if(curStatus == 1 && (count < 2 || count > 3)) {
			matrix[rowIndex][columnIndex] = 2; // 2 denotes that it has moved from Alive to Dead state.
			changed = 1;
		} else if(curStatus == 0 && count == 3) {
			matrix[rowIndex][columnIndex] = 3; // 3 denotes that it has moved from Dead to Alive state.
			changed = 1;
		}
		return changed;
	}

	/**
	 * Returns the count of active neighbours of the given cell in the matrix.
	 * @param rowIndex
	 * @param columnIndex
	 * @param matrix
	 * @return
	 */
	private static int countActiveNeighbours(int rowIndex, int columnIndex, int[][] matrix) {
		int[][] neighbours = {{-1,0},{-1,1},{0,1},{1,1},{1,0},{1,-1},{0,-1},{-1,-1}};
		int count = 0;
		int rows = matrix.length;
		int cols = matrix[0].length;
		for(int[] neighbour : neighbours) {
			int i = rowIndex+neighbour[0];
			int j = columnIndex+neighbour[1];
			if(i >= 0 && i < rows && j >= 0 && j < cols && (matrix[i][j] == 1 || matrix[i][j] == 2))
				count++;
		}
		return count;
	}
}
